@@ -1,4 +1,4 @@
-/* $Id: bogoconfig.c 6713 2008-04-19 03:20:54Z relson $ */
+/* $Id: bogoconfig.c 6811 2009-02-21 20:32:50Z relson $ */
 
 /*****************************************************************************
 
@@ -86,6 +86,7 @@ static void display_tag_array(const char *label, FIELD *array);
 static void process_arglist(int argc, char **argv, priority_t precedence, int pass);
 static bool get_parsed_value(char **arg, double *parm);
 static void comma_parse(char opt, const char *arg, double *parm1, double *parm2, double *parm3);
+static bool token_count_conflict(void);
 
 /*---------------------------------------------------------------------------*/
 
@@ -94,6 +95,7 @@ static struct option longopts_bogofilter[] = {
     LONGOPTIONS_COMMON
     /* longoptions.h - bogofilter */
     LONGOPTIONS_MAIN
+    LONGOPTIONS_MAIN_TUNE
     /* longoptions.h - bogofilter/bogolexer options */
     LONGOPTIONS_LEX
     /* longoptions.h - bogofilter/bogoutil options */
@@ -357,6 +359,9 @@ static const char *help_text[] = {
     "  --terse-format                    short form\n",
     "  --thresh-update                   no update if near 0 or 1\n",
     "  --timestamp                       enable/disable token timestamps\n",
+    "  --token-count                     fixed token count for scoring\n",
+    "  --token-count-min                 min token count for scoring\n",
+    "  --token-count-max                 max token count for scoring\n",
 #ifndef	DISABLE_UNICODE
     "  --unicode                         enable/disable unicode based wordlist\n",
 #endif
@@ -390,7 +395,7 @@ static void print_version(void)
     (void)fprintf(stdout,
 		  "%s version %s\n"
 		  "    Database: %s\n"
-		  "Copyright (C) 2002-2007 David Relson, Matthias Andree\n"
+		  "Copyright (C) 2002-2009 David Relson, Matthias Andree\n"
 		  "Copyright (C) 2002-2004 Greg Louis\n"
 		  "Copyright (C) 2002-2003 Eric S. Raymond, Adrian Otto, Gyepi Sam\n\n"
 		  "%s comes with ABSOLUTELY NO WARRANTY.  "
@@ -473,6 +478,11 @@ static void process_arglist(int argc, char **argv, priority_t precedence, int pa
 	    verbose = max(1, verbose); 	/* force printing */
 	    set_terse_mode_format(inv_terse_mode);
 	}
+	
+	if (token_count_conflict()) {
+	    fprintf(stderr, "Conflicting token count arguments given.\n");
+	    exit(EX_ERROR);
+	}
     }
 
     return;
@@ -725,6 +735,9 @@ int process_arg(int option, const char *name, const char *val, priority_t preced
     case O_TERSE_FORMAT:		terse_format = get_string(name, val);			break;
     case O_THRESH_UPDATE:		get_double(name, val, &thresh_update);			break;
     case O_TIMESTAMP:			timestamp_tokens = get_bool(name, val);			break;
+    case O_TOKEN_COUNT_FIX:             token_count_fix = atoi(val);                            break;
+    case O_TOKEN_COUNT_MIN:             token_count_min = atoi(val);                            break;
+    case O_TOKEN_COUNT_MAX:             token_count_max = atoi(val);                            break;
     case O_UNSURE_SUBJECT_TAG:		unsure_subject_tag = get_string(name, val);		break;
     case O_UNICODE:			encoding = get_bool(name, val) ? E_UNICODE : E_RAW;	break;
     case O_WORDLIST:			configure_wordlist(val);				break;
@@ -746,6 +759,7 @@ int process_arg(int option, const char *name, const char *val, priority_t preced
 
 #define	Q1	if (query >= 1)
 #define	Q2	if (query >= 2)
+#define	Q3	if (query >= 3)
 
 #define YN(b) (b ? "Yes" : "No")
 #define NB(b) ((b != NULL && *b != '\0') ? b : "''")
@@ -762,6 +776,10 @@ rc_t query_config(void)
     Q1 fprintf(stdout, "%-11s = %0.6f  # (%8.2e)\n", "ns_esf", ns_esf, ns_esf);
     Q1 fprintf(stdout, "%-11s = %0.6f  # (%8.2e)\n", "sp_esf", sp_esf, sp_esf);
     Q1 fprintf(stdout, "\n");
+    Q3 fprintf(stdout, "%-17s = %d\n",    "token-count",         token_count_fix);
+    Q3 fprintf(stdout, "%-17s = %d\n",    "token-count-min",     token_count_min);
+    Q3 fprintf(stdout, "%-17s = %d\n",    "token-count-max",     token_count_max);
+    Q3 fprintf(stdout, "\n");
     Q1 fprintf(stdout, "%-17s = %s\n",    "block-on-subnets",    YN(block_on_subnets));
     Q1 fprintf(stdout, "%-17s = %s\n",    "encoding",		 (encoding != E_UNICODE) ? "raw" : "utf-8");
     Q1 fprintf(stdout, "%-17s = %s\n",    "charset-default",     charset_default);
@@ -822,3 +840,20 @@ static void display_tag_array(const char *label, FIELD *array)
 	fprintf(stdout, "%s %s", (i == 0) ? "" : ",", array[i]);
     fprintf(stdout, "\n");
 }
+
+static bool token_count_conflict(void)
+{
+    if (token_count_fix != 0) {
+	if (token_count_fix < token_count_min)
+	    return true;
+    }
+
+    if (token_count_max != 0) {
+	if (token_count_max < token_count_min)
+	    return true;
+	if (token_count_max < token_count_fix)
+	    return true;
+    }
+
+    return false;
+}
