@@ -1,4 +1,4 @@
-/* $Id: bogofilter.c 6714 2008-04-20 15:27:06Z relson $ */
+/* $Id: bogofilter.c 6789 2009-02-12 01:12:53Z relson $ */
 
 /*****************************************************************************
 
@@ -34,6 +34,7 @@ THEORY:
 #include "bogoconfig.h"
 #include "bogoreader.h"
 #include "collect.h"
+#include "format.h"
 #include "passthrough.h"
 #include "register.h"
 #include "rstats.h"
@@ -98,6 +99,8 @@ rc_t bogofilter(int argc, char **argv)
 	wordhash_sort(w);
 	msgcount += 1;
 
+	format_set_counts(w->count, msgcount);
+
         if (!passthrough_keepopen())
             bogoreader_close_ifeof();
         
@@ -110,7 +113,7 @@ rc_t bogofilter(int argc, char **argv)
 
 	if (classify_msg || write_msg) {
 	    lookup_words(w);			/* This reads the database */
-	    spamicity = msg_compute_spamicity(w, NULL);
+	    spamicity = msg_compute_spamicity(w);
 	    status = msg_status();
 	    if (run_type & RUN_UPDATE)		/* Note: don't register if RC_UNSURE */
 	    {
@@ -154,6 +157,8 @@ rc_t bogofilter(int argc, char **argv)
 	register_words(run_type, words, msgcount);
     }
 
+    score_cleanup();
+
     if (logflag && register_opt)
 	write_log_message(status);
 
