@@ -1,4 +1,4 @@
-/* $Id: bogolexer.c 6766 2009-01-12 04:27:36Z relson $ */
+/* $Id: bogolexer.c 6820 2009-02-22 20:13:08Z relson $ */
 
 /*****************************************************************************
 
@@ -32,11 +32,6 @@ AUTHOR:
 
 const char *progname = "bogolexer";
 
-/* prevent larger inclusions */
-
-const char *spam_header_name = SPAM_HEADER_NAME;
-const char *spam_header_place = "";
-
 /* Function Definitions */
 
 static void usage(void)
