@@ -1,4 +1,4 @@
-/* $Id: globals.c 6777 2009-02-01 03:49:48Z relson $ */
+/* $Id: globals.c 6820 2009-02-22 20:13:08Z relson $ */
 
 /*****************************************************************************
 
@@ -57,6 +57,9 @@ const char	*update_dir;
 /*@observer@*/
 const char	*stats_prefix;
 
+const char *spam_header_name = SPAM_HEADER_NAME;
+const char *spam_header_place = "";
+
 /* for lexer_v3.l */
 bool	header_line_markup = true;	/* -H */
 
