@@ -1,4 +1,4 @@
-/* $Id: prob.c 5381 2005-01-17 03:00:48Z relson $ */
+/* $Id: prob.c 6733 2008-06-11 11:15:34Z relson $ */
 
 /*****************************************************************************
 
@@ -16,7 +16,7 @@ AUTHORS:
 
 double calc_prob(uint good, uint bad, uint goodmsgs, uint badmsgs)
 {
-    int n = good + bad;
+    uint n = good + bad;
     double fw, pw;
 
     /* http://www.linuxjournal.com/article.php?sid=6467 */
