@@ -1,4 +1,4 @@
-/* $Id: deqp.c,v 1.5 2004/02/08 15:23:31 relson Exp $ */
+/* $Id: deqp.c 6766 2009-01-12 04:27:36Z relson $ */
 
 #include "common.h"
 
@@ -14,17 +14,21 @@ static void die(void) {
     exit(EXIT_FAILURE);
 }
 
-int main(void) {
+int main(int argc, char **argv) {
     size_t size;
+    qp_mode mode = RFC2045;
     word_t *w;
 
+    if (argc > 1 && strcasecmp(argv[1], "rfc2047")) mode = RFC2047;
+    if (argc > 1 && strcasecmp(argv[1], "rfc-2047")) mode = RFC2047;
+
     if (fseek(stdin, 0, SEEK_END)) die();
     size = ftell(stdin);
     if (fseek(stdin, 0, SEEK_SET)) die();
     w = word_new(NULL, size);
-    if (fread(w->text, 1, w->leng, stdin) != w->leng) die();
-    size = qp_decode(w);
-    if (fwrite(w->text, 1, size, stdout) != size) die();
+    if (fread(w->u.text, 1, w->leng, stdin) != w->leng) die();
+    size = qp_decode(w, mode);
+    if (fwrite(w->u.text, 1, size, stdout) != size) die();
     word_free(w);
     if (fflush(stdout)) die();
     if (fclose(stdout)) die();
