@@ -1,4 +1,4 @@
-/* $Id: wordhash.c 6228 2005-09-04 14:49:43Z relson $ */
+/* $Id: wordhash.c 6816 2009-02-21 21:11:01Z relson $ */
 
 /*
 NAME:
@@ -29,6 +29,7 @@ THEORY:
 #include <string.h>
 #include <stddef.h>	/* for offsetof */
 
+#include "listsort.h"
 #include "wordhash.h"
 #include "xmalloc.h"
 
@@ -75,23 +76,26 @@ static void wh_trap(void) {}
 */
 
 wordhash_t *
-wordhash_init (wh_t t, uint c)
+wordhash_init (wh_t type, uint count)
 {
     wordhash_t *wh = xcalloc (1, sizeof (wordhash_t));
 
-    wh->type = t;
+    wh->type = type;
     wh->count = 0;
-    wh->size = (t == WH_NORMAL) ? 0 : ((c == 0) ? WH_INIT : c);
+    wh->size = (type == WH_NORMAL) ? 0 : ((count == 0) ? WH_INIT : count);
 
-    if (t == WH_NORMAL)
+    switch (type)
+    {
+    case WH_NORMAL:
 	wh->bin = xcalloc (NHASH, sizeof (hashnode_t **));
-
-    if (t == WH_CNTS)
+	break;
+    case WH_CNTS:	// used for bogotune with msg_count files
 	wh->cnts = (wordcnts_t *) xcalloc(wh->size, sizeof(wordcnts_t));
-
-    if (t == WH_PROPS) {
+	break;
+    case WH_PROPS:
 	wh->freeable = true;
 	wh->props = (hashnode_t *) xcalloc(wh->size, sizeof(hashnode_t));
+	break;
     }
 
     return wh;
@@ -100,8 +104,8 @@ wordhash_init (wh_t t, uint c)
 wordhash_t *
 wordhash_new (void)
 {
-    wh_t t = (!fBogotune || !msg_count_file) ? WH_NORMAL : WH_CNTS;
-    wordhash_t *wh = wordhash_init(t, 0);
+    wh_t type = (!fBogotune || !msg_count_file) ? WH_NORMAL : WH_CNTS;
+    wordhash_t *wh = wordhash_init(type, 0);
     return wh;
 }
 
@@ -162,17 +166,23 @@ wordhash_free (wordhash_t *wh)
     wordhash_free_alloc_nodes(wh);
     wordhash_free_strings(wh);
 
-    xfree (wh->order);
-    if (wh->type == WH_CNTS)
+    switch (wh->type)
+    {
+    case WH_NORMAL:
+	break;
+    case WH_CNTS:
 	xfree (wh->cnts);
-    if (wh->type == WH_PROPS) {
+	break;
+    case WH_PROPS:
 	if (wh->freeable) {
 	    uint i;
 	    for (i=0; i<wh->size; i++)
-		xfree(wh->props[i].buf);
+		xfree(wh->props[i].data);
 	}
 	xfree (wh->props);
+	break;
     }
+
     xfree (wh->bin);
     xfree (wh);
 }
@@ -235,15 +245,15 @@ hash (const word_t *t)
     unsigned int h = 0;
     size_t l;
     for (l=0; l<t->leng; l++)
-	h = MULT * h + t->text[l];
+	h = MULT * h + t->u.text[l];
     return h % NHASH;
 }
 
 static void display_node(hashnode_t *n, const char *str)
 {
-    wordprop_t *p = (wordprop_t *)n->buf;
+    wordprop_t *p = (wordprop_t *)n->data;
     if (verbose > 2)
-	printf( "%20.20s %5u %5u%s", n->key->text, p->cnts.bad, p->cnts.good, str);
+	printf( "%20.20s %5u %5u%s", n->key->u.text, p->cnts.bad, p->cnts.good, str);
 }
 
 /* this function accumulates the word frequencies from the src hash to
@@ -263,7 +273,7 @@ void wordhash_add(wordhash_t *dest, wordhash_t *src, void (*initializer)(void *)
     }
 
     for (s = wordhash_first(src); s != NULL; s = wordhash_next(src)) {
-	wordprop_t *p = (wordprop_t *)s->buf;
+	wordprop_t *p = (wordprop_t *)s->data;
 	word_t *key = s->key;
 	wordprop_t *d;
 	if (key == NULL)
@@ -286,7 +296,7 @@ wordhash_foreach (wordhash_t *wh, wh_foreach_t *hook, void *userdata)
     hashnode_t *hn;
 
     for (hn = wordhash_first(wh); hn != NULL; hn = wordhash_next(wh)) {
-	(*hook)(hn->key, hn->buf, userdata);
+	(*hook)(hn->key, hn->data, userdata);
     }
 
     return;
@@ -311,8 +321,8 @@ wordhash_search (const wordhash_t *wh, const word_t *t, unsigned int idx)
 
     for (hn = wh->bin[idx]; hn != NULL; hn = hn->next) {
 	word_t *key = hn->key;
-	if (key->leng == t->leng && memcmp (t->text, key->text, t->leng) == 0) {
-	    wordprop_t *p = (wordprop_t *)hn->buf;
+	if (key->leng == t->leng && memcmp (t->u.text, key->u.text, t->leng) == 0) {
+	    wordprop_t *p = (wordprop_t *)hn->data;
 	    return p;
 	}
     }
@@ -330,11 +340,11 @@ wordhash_standard_insert (wordhash_t *wh, word_t *t, size_t n, void (*initialize
 	return buf;
 
     hn = nmalloc (wh);
-    hn->buf = smalloc (wh, n);
+    hn->data = smalloc (wh, n);
     if (initializer)
-	initializer(hn->buf);
+	initializer(hn->data);
     else
-	memset(hn->buf, '\0', n);
+	memset(hn->data, '\0', n);
 
     hn->key = word_dup(t);
 
@@ -354,7 +364,7 @@ wordhash_standard_insert (wordhash_t *wh, word_t *t, size_t n, void (*initialize
     wh->count += 1;
     wh->size  += 1;
 
-    return hn->buf;
+    return hn->data;
 }
 
 static void *
@@ -396,18 +406,14 @@ wordhash_first (wordhash_t *wh)
     case WH_NORMAL:
 	val = wh->iter_ptr = wh->iter_head;
 	break;
-    case WH_ORDERED:
+    case WH_CNTS:
 	wh->index = 0;
-	val = wh->order[wh->index];
+	val = &wh->cnts[wh->index];
 	break;
     case WH_PROPS:
 	wh->index = 0;
 	val = &wh->props[wh->index];
 	break;
-    case WH_CNTS:
-	wh->index = 0;
-	val = &wh->cnts[wh->index];
-	break;
     }
 
     return val;
@@ -423,35 +429,32 @@ wordhash_next (wordhash_t *wh)
 	if (wh->iter_ptr != NULL)
 	    val = wh->iter_ptr = wh->iter_ptr->iter_next;
 	break;
-    case WH_ORDERED:
+    case WH_CNTS:
 	if (++wh->index < wh->count)
-	    val = wh->order[wh->index];
+	    val = &wh->cnts[wh->index];
 	break;
     case WH_PROPS:
 	if (++wh->index < wh->count)
 	    val = &wh->props[wh->index];
 	break;
-    case WH_CNTS:
-	if (++wh->index < wh->count)
-	    val = &wh->cnts[wh->index];
-	break;
     }
 
     return val;
 }
 
-static int compare_hashnode_t(const void *const ihn1, const void *const ihn2)
+/* compare_hashnode_t - sort by ascending token text */
+
+static int compare_hashnode_t(const void *const pv1, const void *const pv2)
 {
-    const hashnode_t *hn1 = *(const hashnode_t *const *)ihn1;
-    const hashnode_t *hn2 = *(const hashnode_t *const *)ihn2;
-    int   cmp = word_cmp(hn1->key, hn2->key);
-    return cmp;
+    const hashnode_t *hn1 = (const hashnode_t *)pv1;
+    const hashnode_t *hn2 = (const hashnode_t *)pv2;
+    return word_cmp(hn1->key, hn2->key);
 }
 
 static wordcnts_t *wordhash_get_counts(wordhash_t *wh, hashnode_t *n)
 {
     if (wh->cnts == NULL) {
-	wordprop_t *p = (wordprop_t *)n->buf;
+	wordprop_t *p = (wordprop_t *)n->data;
 	wordcnts_t *c = &p->cnts;
 	return c;
     }
@@ -473,25 +476,12 @@ wordhash_set_counts(wordhash_t *wh, int good, int bad)
     }
 }
 
+/* wordhash_sort - sort by ascending token text */
+
 void
 wordhash_sort (wordhash_t *wh)
 {
-    hashnode_t *node;
-
-    if (wh->size == 0
-	|| wh->type != WH_NORMAL
-	|| msg_count_file)
-	return;
-
-    wh->order = (hashnode_t **) xcalloc(wh->size, sizeof(hashnode_t *));
-
-    wh->count = 0;
-    for (node = wordhash_first(wh); node != NULL; node = wordhash_next(wh))
-	wh->order[wh->count++] = node;
-
-    qsort(wh->order, wh->count, sizeof(hashnode_t *), compare_hashnode_t);
-
-    wh->type = WH_ORDERED;
+    wh->iter_head = (hashnode_t *)listsort((element *)wh->iter_head, (fcn_compare *)&compare_hashnode_t);
 
     return;
 }
@@ -556,11 +546,11 @@ convert_wordhash_to_propslist(wordhash_t *whi, wordhash_t *db)
 	    }
 	    else {
 		wp = xcalloc(1, sizeof(wordprop_t));
-		memcpy(wp, node->buf, sizeof(wordprop_t));
+		memcpy(wp, node->data, sizeof(wordprop_t));
 		if (!who->freeable)
 		    wh_trap();
 	    }
-	    who->props[who->count].buf = wp;
+	    who->props[who->count].data = wp;
 	    xfree(node->key);
 	    node->key = NULL;
 	    who->count += 1;
