@@ -1,4 +1,4 @@
-/* $Id: wordhash.h 3997 2004-01-31 00:09:24Z relson $ */
+/* $Id: wordhash.h 6799 2009-02-14 22:10:54Z relson $ */
 
 #ifndef WORDHASH_H
 #define WORDHASH_H
@@ -7,10 +7,10 @@
 
 /* Hash entry. */
 typedef struct hashnode_t {
-  word_t *key;					/* word key */
-  void   *buf;					/* Associated buffer. To be used by caller. */
-  struct hashnode_t *next;			/* Next item in linked list of items with same hash */ 
   /*@dependent@*/ struct hashnode_t *iter_next;	/* Next item added to hash. For fast traversal */
+  struct hashnode_t *next;			/* Next item in linked list of items with same hash */ 
+  word_t *key;					/* word key */
+  void   *data;					/* Associated data. To be used by caller. */
 } hashnode_t;
 
 typedef struct wh_alloc_node {
@@ -29,7 +29,9 @@ typedef struct wh_alloc_str {
 
 typedef /*@null@*/ hashnode_t *hashnode_pt;
 
-typedef enum wh_e { WH_NORMAL, WH_ORDERED, WH_PROPS, WH_CNTS } wh_t;
+typedef enum wh_e { WH_NORMAL,
+		    WH_PROPS,
+		    WH_CNTS } wh_t;
 
 typedef struct wordhash_s {
   /*@null@*/  /*@dependent@*/ wh_t type;		/* normal, ordered, props, or cnts */
@@ -39,20 +41,19 @@ typedef struct wordhash_s {
   /*@null@*/  /*@dependent@*/ uint size;		/* size of array */
 
   hashnode_pt *bin;
-  /*@null@*/ /*@owned@*/ wh_alloc_node *nodes;		/*list of node buffers */
+  /*@null@*/ /*@owned@*/ wh_alloc_node *nodes;		/* list of node buffers */
   /*@null@*/  		 wh_alloc_str  *strings;	/* list of string buffers */
 
   /*@null@*/  /*@dependent@*/ hashnode_t *iter_ptr;
   /*@null@*/  /*@dependent@*/ hashnode_t *iter_head;
   /*@null@*/  /*@dependent@*/ hashnode_t *iter_tail;
 
-  /*@null@*/  /*@dependent@*/ hashnode_t **order;	/* array of nodes */
-  /*@null@*/  /*@dependent@*/ hashnode_t  *props;	/* array of nodes */
-  /*@null@*/  /*@dependent@*/ wordcnts_t  *cnts;	/* array of counts */
+  /*@null@*/  /*@dependent@*/ hashnode_t *props;	/* array of nodes */
+  /*@null@*/  /*@dependent@*/ wordcnts_t *cnts;		/* array of counts */
 } wordhash_t;
 
 /*@only@*/ wordhash_t *wordhash_new(void);
-/*@only@*/ wordhash_t *wordhash_init(wh_t t, uint c);
+/*@only@*/ wordhash_t *wordhash_init(wh_t type, uint count);
 
 void wordhash_free(/*@only@*/ wordhash_t *);
 size_t wordhash_count(wordhash_t * h);
